// Programmer: Maura Dailey
// CSE 120 Section #: 1
// Date: 03/25/02

// Brief: This header file sets forth the prototypes for
//   a Sorted List ADT. 

#include "ItemType.h"
// ItemType.h defines ItemType and NodeType which must be 
// templated with template parameter ItemType. ItemType.h
// should be edited to reflect the variable type for the
// nodes in this Sorted List ADT.
//
// ItemType must be a type for which the operators "<" 
// and "==" are defined, either a primitive type or
// a class that overloads these operators.

template <class ItemType>
class SortedType
{
  public:

    SortedType<ItemType>();
    // Constructor that sets the list to its initial state
    // Post: List has been created, length is zero.

    ~SortedType<ItemType>();
    // Destructor that deletes the list and frees memory
    // Pre: List has been created
    // Post: List has been destroyed

    bool IsFull() const;
    // Determines whether list is full.
    // Post: Function value = (list is full)

    int  LengthIs() const;
    // Determines the number of elements in list.
    // Post: Function value = number of elements in list.

    void MakeEmpty();
    // Initializes list to empty state.
    // Post:  List is empty.

    void RetrieveItem(ItemType& item, bool& found);
    // Retrieves list element whose key matches item's key 
    // (if present).
    // Pre:  Key member of item is initialized.
    // Post: If there is an element someItem whose key matches 
    //       item's key, then found = true and item is a copy 
    //       of someItem; otherwise found = false and item is 
    //       unchanged. 
    //       List is unchanged.

    void InsertItem(ItemType item); 
    // Adds item to list.
    // Pre:  List is not full.
    //       item is not in list. 
    // Post: item is in list 
    //       list order is preserved

    void DeleteItem(ItemType item);
    // Deletes the element whose key matches item's key.
    // Pre:  Key member of item is initialized.
    //       One and only one element in list has a key matching
    //       item's key.
    // Post: No element in list has a key matching item's key.

    void ResetList();
    // Initializes current position to beginning of list for iteration 
    // through list.
    // Post: Current position is at beginning of list.

    void GetPrevItem(ItemType& item, bool& found);
    // Gets the previous item in the list.
    // Pre: Current position is defined.
    // Post: Current position is updated to previous position.
    //       item is a copy of element at previous position unless
    //       current position was the beginning of the list, in which
    //       case found is set to false.

    void GetNextItem(ItemType& item, bool& found);
    // Gets the next item in the list.
    // Pre: Current position is defined.
    // Post: Current position is updated to next position.
    //       item is a copy of element at current position unless
    //       current position was the end of the list, in which case
    //       found is set to false.

    SortedType(const SortedType<ItemType>&);
    // Overloaded Constructor required for the overloaded assignment
    // operator.

    void operator= (SortedType<ItemType>);
    // Overloaded assignment operator, used to exactly copy one
    // list into another.
    // Post: newList has been assigned to the list variable on the
    //       left side of the assignment operator.

  private:
    // listData: pointer to the first item in a list
    NodeType<ItemType>* listData;

    // length: incremented or decremented to keep track of the length of
    //   the list
    int length;

    // currentPos: pointer to the current node inside the list
    NodeType<ItemType>* currentPos;
};

#include "SortedType.cpp"