// Programmer: Maura Dailey
// CSE 120 Section #: 1
// Date: 03/25/02

// Brief: The purpose of this program is to test our Sorted, doubly
//   linked list ADT. Several tests will be run to determine that each
//   function works as expected in a variety of situations. The results
//   of these tests will be displayed to the screen and to an output
//   file, accompanied by relevant documentation.

// Test Cases: We will test several situations that may occur during the use
// of our Sorted Doubly Linked List ADT. Among them will be the following:

//  1) We will first test the initialization of the list data type by
//     initializing the data type, then by calling LengthIs() to determine that
//     the list has been properly made and the length set to zero.
//  2) Next, we need to test our InsertItem() function by adding names to our list
//     data type. Output will be generated to inform the user as each name is
//     added.
//  3) The LengthIs() function is called again to determine that the lenghth of
//     the list has been properly incremented.
//  4) After that, we set the current position inside the list to be at the
//     beginning and utilize a loop to get each item using the GetNextItem()
//     function. Output will be generated to demonstrate that each item has been
//     correctly retrieved.
//  5) Next, we test the GetPrevItem() function by using the same process as our
//     test for GetNextItem(). Output will be generated to demonstrate that each
//     item has been correctly retrieved.
//  6) Next our list data type will be assigned to newly created list data type,
//     the current position inside the list will be reset to the beginning with
//     ResetList(), and the contents displayed using our GetNextItem() function.
//  7) Now we test our RetrieveItem() function by searching for a name from the list.
//     Once that name has been found, we then test our DeleteItem() function by
//     removing it.
//  8) The resulting modified list will then be redisplayed using the GetNextItem()
//     function to demonstrate that the item has been deleted correctly.
//  9) This modified list is then copied back over the original list data type we
//     began testing with. The contents of this original list are then displayed to
//     demonstrate that the list has been overwritten.

#include "SortedType.h"
#include <iostream>
#include <fstream>
#include <string>

using namespace std;

int main()
{
  // The list data type has been created, and thus initialized (Test Case 1)
  SortedType<string> stringListA;
  string Item1, Item2;

  ofstream outFile;
  outFile.open("testPlan.dat");

  bool found;

  cout << "Creating stringListA, a list variable of type string that will" << endl
       << "use our Sorted Doubly Linked List ADT." << endl << endl;

  outFile << "stringListA has been created and initialized to its initial values."
          << endl << endl;

  // The LengthIs() function is called to verify that the list data type has been
  // initialized. (Test Case 1)

  cout << "As a newly created list, the length of stringListA should be zero. The"
       << "initial length of the newly created list stringListA is: " << stringListA.LengthIs()
       << endl << endl;

  outFile << "Provided the newly created doubly linked list has been properly" << endl
          << "initialized, the length of the list should be zero. Length is: " << stringListA.LengthIs()
          << endl << endl;

  cout << "Now we will fill the list with several values." << endl << endl;

  outFile << "Now we will insert several names into our list data type" << endl << endl;

  // InsertItem() is tested by adding several names in succession to the list data
  // type (Test Case 2)

  cout << "Inserting the name \"Katie\"..." << endl;
  outFile << "Inserting the name \"Katie\" ..." << endl;
  Item1 = "Katie";
  stringListA.InsertItem(Item1);

  cout << "Inserting the name \"Alisha\"..." << endl;
  outFile << "Inserting the name \"Alisha\" ..." << endl;
  Item1 = "Alisha";
  stringListA.InsertItem(Item1);

  cout << "Inserting the name \"Justin\"..." << endl;
  outFile << "Inserting the name \"Justin\" ..." << endl;
  Item1 = "Justin";
  stringListA.InsertItem(Item1);

  cout << "Inserting the name \"Vance\"..." << endl;
  outFile << "Inserting the name \"Vance\" ..." << endl;
  Item1 = "Vance";
  stringListA.InsertItem(Item1);

  cout << "Inserting the name \"Bob\"..." << endl << endl;
  outFile << "Inserting the name \"Bob\" ..." << endl << endl;
  Item1 = "Bob";
  stringListA.InsertItem(Item1);

  // The length of the list will be displayed using the LengthIs() function to
  // determine that the list is the proper length (Test Case 3)

  cout << "Now that we have inserted 5 names into our list, it is" << endl
       << "necessary to check that the length of the list has been" << endl
       << "properly incremented. The length of the list is now: "
       << stringListA.LengthIs() << endl << endl;

  outFile << "We again check the length of the list to determine if the length" << endl
          << "has been properly incremented. Length is now: " << stringListA.LengthIs()
          << endl << endl;

  cout << "Now we will reset the list so that the current position" << endl
       << "is at the beginning of the list." << endl << endl;

  outFile << "We will now call the function ResetList() to set the current postion" << endl
          << "inside the list to the beginning." << endl << endl;

  // ResetList() is called to reset the current position inside the list data type to
  // be at the beginning (Test Case 4)

  stringListA.ResetList();

  cout << "While we have not yet reached the end of the list, we will" << endl
       << "get the next item and display it to the screen to prove that" << endl
       << "the data inside the list is in sorted order." << endl << endl;

  outFile << "We will now progress through the list and display each item to" << endl
          << "demonstrate that the contents are indeed in sorted order." << endl << endl;

  // GetNextItem() is used to retrieve each item in the list, which is then displayed
  // to demonstrate that the list maintains its contents in sorted order (Test Case 4)

  for(int i = 1; i <= stringListA.LengthIs(); i++)
  {
    found = false;
    stringListA.GetNextItem(Item2, found);
    if (found)
    {
      cout << Item2 << " has been retrieved." << endl;
      outFile << Item2 << " has been retrieved." << endl;
    }
  }

  cout << endl << "We will now travel backwards through the list to demonstrate that"
       << endl << "we can traverse the list backwards." << endl << endl;

  outFile << endl << "Next, we travel backwards through the list to demonstrate that"
          << endl << "our GetPrevItem() function works as expected." << endl << endl;

  // GetPrevItem() is used to retrieve each item in the list in reverse order, which are
  // then displayed (Test Case 5)

  for(int i = 1; i <= stringListA.LengthIs(); i++)
  {
    found = false;
    stringListA.GetPrevItem(Item2, found);
    if (found)
    {
      cout << Item2 << " has been retrieved." << endl;
      outFile << Item2 << " has been retrieved." << endl;
    }
  }

  cout << endl << "Now we will copy the contents of the current list to a new list"
       << endl << "stringListB and reset the list so that the current position is at"
       << endl << "the beginning." << endl << endl;

  outFile << endl << "Now we assign stringListA to stringListB and reset the current"
          << endl << "position inside the list to be at the beginning." << endl << endl;

  // A new list data type is created, and the contents of the last list data type
  // assigned to it (Test Case 6)

  SortedType<string> stringListB;
  stringListB = stringListA;

  stringListB.ResetList();

  cout << "We will now display the contents of stringListB to demonstrate it" << endl
       << "contains the same data." << endl << endl;

  outFile << "We will now display the contents of stringListB to demonstrate" << endl
          << "that it now contains the same data as stringListA." << endl << endl;

  // The contents of the new list data type will be displayed to demonstrate that
  // it is an exact copy of the old list data type (Test Case 6)

  for(int i = 1; i <= stringListB.LengthIs(); i++)
  {
    found = false;
    stringListB.GetNextItem(Item2, found);
    if (found)
    {
      cout << Item2 << " has been retrieved." << endl;
      outFile << Item2 << " has been retrieved." << endl;
    }
  }

  cout << endl << "We will now search through the list for the name \"Bob\" and then"
       << endl << "delete it from the list." << endl << endl;

  outFile << endl << "Now we test the DeleteItem() function by first retrieving the"
          << endl << "name \"Bob\" from the list, then by calling the DeleteItem()"
          << endl << "function to remove the name." << endl << endl;

  Item2 = "Bob";
  found = false;

  // RetrieveItem() attempts to find an item inside of the list, and if found, DeleteItem()
  // is called to remove it (Test Case 7)

  stringListB.RetrieveItem(Item2, found);
  if (found)
  {
    stringListB.DeleteItem(Item2);
  }

  cout << "To prove that the item has been deleted, we will now redisplay the" << endl
       << "contents of stringListB." << endl << endl;

  outFile << "To demonstrate that the name has really been deleted, the contents" << endl
          << "of stringListB will again be displayed." << endl << endl;

  stringListB.ResetList();

  // GetNextItem() is used to retrieve each item of the newly modified list in an effort
  // to determine if the item has been deleted (Test Case 8)

  for(int i = 1; i <= stringListB.LengthIs(); i++)
  {
    found = false;
    stringListB.GetNextItem(Item2, found);
    if (found)
    {
      cout << Item2 << " has been retrieved." << endl;
      outFile << Item2 << " has been retrieved." << endl;
    }
  }

  cout << endl << "We will now copy back the modified list to stringListA and display"
       << endl << "the contents of stringListA once more" << endl << endl;

  outFile << endl << "stringListB will now be copied back to stringListA, erasing its"
          << endl << "original values. stringListA will then be redisplayed, to prove"
          << endl << "that it is a copy of stringListB." << endl << endl;

  // The now modified list is then copied over the original list data type created in
  // this program (Test Case 9)

  stringListA = stringListB;
  stringListA.ResetList();

  // GetNextItem() is used to get each item in the list in an effort to show that the
  // copy was completed successfully (Test Case 9)

  for(int i = 1; i <= stringListA.LengthIs(); i++)
  {
    found = false;
    stringListA.GetNextItem(Item2, found);
    if (found)
    {
      cout << Item2 << " has been retrieved." << endl;
      outFile << Item2 << " has been retrieved." << endl;
    }
  }

  cout << endl << "The new length of stringListA is: " << stringListA.LengthIs()
       << endl << endl;

  outFile << endl << "The new length of stringListA is: " << stringListA.LengthIs()
          << endl << endl;

  cout << "Testing has been completed." << endl;

  outFile << "Testing has been completed." << endl;
  
  outFile.close();
  return 0;
}